# -*- coding: utf-8 -*-
"""views module for logging_endpoint package.

Copyright 2019 Propylon Ltd.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from __future__ import unicode_literals

import json
import logging

from django.http import HttpResponse
from django.utils.module_loading import import_string

import six
from logging_endpoint.settings import LOGGER, MESSAGE_HANDLER

logger = logging.getLogger(LOGGER)
if isinstance(MESSAGE_HANDLER, six.string_types):
    MESSAGE_HANDLER = import_string(MESSAGE_HANDLER)

try:
    from json.decoder import JSONDecodeError
except ImportError:
    JSONDecodeError = ValueError


def get_messages(request):
    """Return the messages in the given request.

    If the body is json deserializable, a list is decomposed to individual
    messages.
    """
    try:
        content = json.loads(request.body)
        if isinstance(content, list):
            return content

        return [content]
    except JSONDecodeError:
        return [request.body]


def logging_view(request):
    """Send the received logs to the logger."""
    for message in get_messages(request):
        msg_logger, level, msg, args, kwargs = MESSAGE_HANDLER(message)
        if not msg_logger:
            msg_logger = logger

        msg_logger.log(level, msg)

    return HttpResponse(status=200)



# -*- coding: utf-8 -*-
"""message handler module for logging_endpoint package.

Copyright 2019 Propylon Ltd.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from logging import INFO, getLevelName

from logging_endpoint.settings import OVERWRITE_LOGGER


def default_handler(log_data):
    """Return the message as is as level INFO on the default logger."""
    if isinstance(log_data, bytes):
        log_data = log_data.decode()
    return None, INFO, str(log_data), tuple(), dict()


def json_handler(log_data):
    """Return the message based on a json dictionary.

    Example input:
    {
        'message':   'my log message',
        'logger':    'user interaction',
        'loglevel':  'error',
        'timestamp': '2020-01-01T12:00Z'
    }

    Results in:
    * Log message of the given loglevel
    * If the setting OVERWRITE_LOGGER is set, the given logger is used.
      Otherwise the standard logger is used, but the given one is added to the
      complete message
    * The timestamp is added to the log message

    -> ERROR - 2020-01-01T12:00Z - user interaction - my log message
    """
    logger = log_data.get('logger', None)
    level = getLevelName(log_data.get('loglevel', 'INFO').upper())
    msg = '-'.join(filter(None, (
        log_data.get('timestamp', ''),
        logger,
        str(log_data.get('message', None))
    )))
    args = tuple()
    kwargs = dict()

    return (
        logger if logger and OVERWRITE_LOGGER else None,
        level,
        msg,
        args,
        kwargs
    )



"""
.. module:: setup
   :synopsis: Installation information for django-logging-endpoint
.. moduleauthor:: Propylon
"""
# Standard
import os
from setuptools import setup, find_packages

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

INSTALL_REQUIRES = ['setuptools', 'Django>=1.11', 'six']


setup(
    name='django-logging-endpoint',
    version='1.0.1',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    include_package_data=True,
    license='Apache-2.0',
    description='Provide an endpoint to receive logs and push them to a '
        'configurable django logger',
    long_description=README,
    url='',
    author='Propylon',
    author_email='opensource@propylon.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.2',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    install_requires=INSTALL_REQUIRES,
)
